# 自訂規則
授人以魚，不如授人以漁。Dism++ 的能力是有限的，無法覆蓋的所有使用者的需求。因此給程式新增了自訂規則功能，進階用戶打造自己的專屬工具。

## 建立一個空白的自訂規則檔案
我們只需要在 Config 目錄新增一個 Custom\*.xml 這樣形式的 xml 檔案即可（\*表示任何字串，比如 Custom1.xml 或者 Custom我愛我的家.xml，另外檔案以 UTF8 或者 Unicode 編碼儲存）。初始檔案內容如下：
```xml
<Data>
  <CleanCollection4>
  <!--在此處編寫垃圾清理規則-->
  </CleanCollection4>

  <SystemOptimization>
  <!--在此處編寫系統最佳化規則-->
  </SystemOptimization>
</Data>
```
等所有工作完成後，在將這個 xml 檔案壓縮成一個 zip，比如 Custom1.zip 或者 Custom我愛我的家.zip。
> 壓縮為 zip 這是為了減少檔案容量，避免造成不必要的檔案佔用。

## 垃圾清理規則從負五開始
下面我們將循序漸進，讓你從入門到放棄。說錯了，是入門到精通。當然也歡迎大家提供寶貴的意見。

### 編寫一個最簡單的清理規則
我們都知道 C:\\Windows\\Logs 是系統的日誌目錄。系統執行時會在此處產生大量日誌，而且某些情況還不會自動刪除。因此我們需要定期刪除此檔案。那我們如何透過規則來實現此功能呢？

```xml
<Data>
  <CleanCollection4>
    <!--Name 表示清理規則的名稱-->
    <Item Name="系統 Logs 目錄">
      <!--Description 節表示清理規則的描述-->
      <Description>系統的日誌目錄，用於存放系統執行日誌。</Description>
      <!--Group 節表示規則的分組，同一個分組儲存類似的規則-->
      <Group>臨時檔案</Group>
      <Scan>
        <Activate>
          <!--General 規則，清空 C:\Windows\Logs 資料夾下方的所有內容，但是不會刪除 Logs 資料夾本身-->
          <General RootPath="C:\Windows\Logs"/>
        </Activate>
      </Scan>
    </Item>
  </CleanCollection4>
</Data>
```
我想，聰明的你一定明白了，大概怎麼寫規則了。至於 General 的完整用法，可以參考引擎白皮書。

### 編寫帶環境變數發清理規則中
之前的規則你一定發現一個嚴重的問題。就是清理路徑是死的。就只能是清空 `C:\Windows\Logs` 目錄。系統槽並不一定是 C……，因此這個規則是有瑕疵的。因此我們有了改進版。使用環境變數。

```xml
<!--改良版-->
<Data>
  <CleanCollection4>
    <Item Name="系統 Logs 目錄">
      <Description>系統的日誌目錄，用於存放系統執行日誌。</Description>
      <Group>臨時檔案</Group>
      <Scan>
        <Activate>
          <General RootPath="%SystemRoot%\Logs"/>
        </Activate>
      </Scan>
    </Item>
  </CleanCollection4>
</Data>
```
%SystemRoot% 就是一個環境變數，表示 C:\\Windows。具體所支援的環境變數，請參考後續文件——環境變數。現在規則適應性大大提高了。

### 編寫多 General 清理規則
我們知道系統相關日誌不止一個地方。我們可以把他們收錄在一個規則裡。比如 Windows 目錄還有各種 log 檔案。比如 WindowsUpdate.log、setuperr.log、setupact.log 等等。General 語法提供了 Query 功能可以刪除他們。

```xml
<Data>
  <CleanCollection4>
    <Item Name="系統 Logs 目錄">
      <Description>系統的日誌目錄，用於存放系統執行日誌。</Description>
      <Group>臨時檔案</Group>
      <Scan>
        <Activate>
          <General RootPath="%SystemRoot%\Logs"/>

          <!--General 枚舉語法，枚舉 %SystemRoot% 中的檔案，然後刪除 WindowsUpdate.log、setuperr.log、setupact.log 3 個檔案-->
          <!--Flags 是個篩選器，支援的值為 File 以及 Directory，如果沒有 Flags 則同時枚舉檔案以及資料夾。在此處僅枚舉檔案-->
          <General RootPath="%SystemRoot%" Flags="File">
            <Query>WindowsUpdate.log</Query>
            <Query>setuperr.log</Query>
            <Query>setupact.log</Query>
          </General>
        </Activate>
      </Scan>
    </Item>
  </CleanCollection4>
</Data>
```
### 使用通配改進清理規則
你一定感覺到這個規則太蠢了，如果後來又新增加了 log 檔案怎麼辦？其實 Query 是支援簡單的萬用字元的。將 後綴是 .log 的檔案全部刪除即可。因此我們可以使用通配來調整規則。

```xml
<Data>
  <CleanCollection4>
    <Item Name="系統 Logs 目錄">
      <Description>系統的日誌目錄，用於存放系統執行日誌。</Description>
      <Group>臨時檔案</Group>
      <Scan>
        <Activate>
          <General RootPath="%SystemRoot%\Logs"/>
          <!--General 枚舉語法，枚舉 %SystemRoot% 中的檔案，然後刪除所有符合 *.log 的檔案-->
          <General RootPath="%SystemRoot%" Flags="File">
            <Query>*.log</Query>
          </General>
        </Activate>
      </Scan>
    </Item>
  </CleanCollection4>
</Data>
```
但是通配也會有問題，假如我們不希望 Dism++ 刪除 %SystemRoot%\\WindowsUpdate.log。我們可以使用 Excluded 排除不想刪除的內容，當然 Excluded 也支援通配。
```xml
<Data>
  <CleanCollection4>
    <Item Name="系統 Logs 目錄">
      <Description>系統的日誌目錄，用於存放系統執行日誌。</Description>
      <Group>臨時檔案</Group>
      <Scan>
        <Activate>
          <General RootPath="%SystemRoot%\Logs"/>
          <General RootPath="%SystemRoot%" Flags="File">
            <Query>*.log</Query>
            <!--排除 WindowsUpdate.log-->
            <Excluded>WindowsUpdate.log</Excluded>
          </General>
        </Activate>
      </Scan>
    </Item>
  </CleanCollection4>
</Data>
```
