# 規則結構參考
本文內容用於詳細說明 Dism++ 規則的組成，不感興趣的同學可以無視本文內容。

## 清理規則結構參考
很多朋友可能還是有疑問，Dism++ 的規則形式到底是怎麼樣的。本文將詳細說明清理規則完整語法。清理規則有 V1 版跟 V2 版，這是由於歷史原因導致的。Dism++ 同時支援二個版本，可以根據自己的需要選擇。

### 清理規則結構體 V1
V1 版結構體會簡潔很多
```xml
<!--V1 版結構體-->
<element name="Item">
  <!--規則的名稱-->
  <attribute name="Name" type="字串" use="必選"/>
  <!--規則安全等級，0~3 的整數，數字越大表示越安全。他們分別表示 存在風險、建議保留、可以刪除、建議刪除 4 個等級
  存在風險級別的項目，在新手模式不顯示，在專家模式中以橙色底色突出項目。
  建議保留、可以刪除 是屬於同一個級別，Dism++ 預設不清理。這是歷史原因，請大家不要糾結為什麼有二個等級是一樣的作用。
  建議刪除 的項目 Dism++ 將預設清理。使用此等級一定要確保規則的安全性，一般不推薦使用。-->
  <attribute name="Level" type="整數" use="選用" default="2"/>

  <!--規則的描述-->
  <element name="Description" type="字串" use="選用"/>

  <element name="Scan">
    <complexType>
      <!--任務的進度比重，一個大於 0 的整數。在計算進度時有用-->
      <attribute name="Work" type="整數" use="選用" default="1"/>

      <!--警告文字，使用者啟用此規則時的彈窗說明。-->
      <element name="Warning" type="字串" use="選用"/>
      <!--分組名稱，如果不存在此節點，則統一歸類到「其他」類別-->
      <element name="Group" type="字串" use="選用" default="其他"/>
      <!--適應性檢測規則，比如某規則僅在 8.1 以上平台顯示，可以使用適應性檢測規則-->
      <element name="Applicable" type="XML" use="選用" default="<True/>"/>

      <element name="Activate">
        <complexType>
          <!--執行清理或者掃描時，是否重新檢測適應性規則。適應性及時性要求較高項目可以使用。-->
          <attribute name="Rescan" type="邏輯值" use="選用"/>
		  <!--10.1.25.2 開始支援，執行動作時，重新啟動相關軟體。值目前僅能輸入 Explorer-->
		  <attribute name="Restart" type="字串" use="選用"/>
          <sequence>
            <!--在此處填寫動作執行引擎支援的規則-->
          </sequence>
        </complexType>
      </element>
    </complexType>
  </element>
</element>
```

### 清理規則結構體 V2
V2 版結構體引入了 ScanCollection，支援多個 Scan 節點。
```xml
<!--V2 版結構體-->
<element name="Item">
  <!--規則的名稱-->
  <attribute name="Name" type="字串" use="必選"/>
  <!--規則安全等級，0~3 的整數，數字越大表示越安全。他們分別表示 存在風險、建議保留、可以刪除、建議刪除 4 個等級
  存在風險級別的項目，在新手模式不顯示，在專家模式中以橙色底色突出項目。
  建議保留、可以刪除 是屬於同一個級別，Dism++ 預設不清理。這是歷史原因，請大家不要糾結為什麼有二個等級是一樣的作用。
  建議刪除 的項目 Dism++ 將預設清理。使用此等級一定要確保規則的安全性，一般不推薦使用。-->
  <attribute name="Level" type="整數" use="選用" default="2"/>

  <!--規則的描述-->
  <element name="Description" type="字串" use="選用"/>
  <element name="ScanCollection">
    <sequence>
      <element name="Scan">
        <complexType>
          <!--任務的進度比重，一個大於 0 的整數。在計算進度時有用-->
          <attribute name="Work" type="整數" use="選用" default="1"/>

          <!--規則的名稱，作用跟 Item 節點的 Name 屬性相同，優先使用 Scan 中的 Name-->
          <element name="Name" type="字串" use="選用"/>
          <!--規則的描述，作用跟 Item 屬性中的 Description 相同，只是優先度更高-->
          <element name="Description" type="字串" use="選用"/>
          <!--警告文字，使用者啟用此規則時的彈窗說明。-->
          <element name="Warning" type="字串" use="選用"/>
          <!--分組名稱，如果不存在此節點，則統一歸類到「其他」類別-->
          <element name="Group" type="字串" use="選用" default="其他"/>
          <!--適應性檢測規則，比如某規則僅在 8.1 以上平台顯示，可以使用適應性檢測規則-->
          <element name="Applicable" type="XML" use="選用" default="<True/>"/>

          <element name="Activate">
            <complexType>
              <!--執行清理或者掃描時，是否重新檢測適應性規則。適應性及時性要求較高項目可以使用。-->
              <attribute name="Rescan" type="邏輯值" use="選用"/>
			  <!--10.1.25.2 開始支援，執行動作時，重新啟動相關軟體。值目前僅能輸入 Explorer-->
			  <attribute name="Restart" type="字串" use="選用"/>
              <sequence>
                <!--在此處填寫動作執行引擎支援的規則-->
              </sequence>
            </complexType>
          </element>
        </complexType>
      </element>
    </sequence>
  </element>
</element>
```
## 系統最佳化規則結構參考



## 執行引擎

### RegWrite
執行登錄檔寫入
```xml
<!--
1：建立登錄檔 HKEY_LOCAL_MACHINE\SOFTWARE\Test 並這個鍵中建立一個名字叫 123 的 REG_DOWRD 類型，值為 1 的值。
      <RegWrite Key=”HKEY_LOCAL_MACHINE\SOFTWARE\Test” Type=”REG_DWORD” Value=”123” Data=”1”/>

2：建立登錄檔 HKEY_LOCAL_MACHINE\SOFTWARE\Test 並在子健中建立名字叫 123，類型為 REG_BINARY，值為  0xCC 0x10
      <RegWrite Key=”HKEY_LOCAL_MACHINE\SOFTWARE\Test” Type=” REG_BINARY” Value=”123” Data=”CC10”/>
-->
<element name="RegWrite">
  <complexType>
    <!--登錄檔路徑，比如 "HKEY_LOCAL_MACHINE\SOFTWARE\Test"-->
    <attribute name="Key" type="字串" use="必選"/>
    <!--登錄檔類型，可以是 REG_SZ、REG_DWORD、REG_QWORD、REG_BINARY、REG_EXPAND_SZ-->
    <attribute name="Type" type="字串" use="選用"/>
    <!--值的名稱-->
    <attribute name="Value" type="字串" use="選用"/>
    <!--值的資料-->
    <attribute name="Data" type="字串" use="選用"/>
    <!--操作類型，可以是 =、|、&以及^-->
    <attribute name="Operator" type="字串" use="選用" default="="/>
    <!--預設值。存在 Operator（|、&、^）時必須填寫預設值。-->
    <attribute name="Default" type="字串" use="選用"/>
    <!--True 或者 False，指示是否開啟 Wow64 重定向，32 位元系統自動忽略-->
    <attribute name="Wow64" type="字串" use="選用" default="False"/>
  </complexType>
</element>      
```
#### RegDelete
執行登錄檔刪除

```xml
<element name="RegDelete">
  <complexType>
    <!--登錄檔路徑，比如 "HKEY_LOCAL_MACHINE\SOFTWARE\Test"-->
    <attribute name="Key" type="字串" use="必選"/>
    <!--值的名稱。如果不存在此屬性那麼刪除整個登錄檔樹，如果存在，那麼僅刪這個值。-->
    <attribute name="Value" type="字串" use="選用"/>
    <!--True 或者 False，指示是否開啟 Wow64 重定向，32 位元系統自動忽略-->
    <attribute name="Wow64" type="字串" use="選用" default="False"/>
  </complexType>
</element>
<!--
1：刪除整個 HKEY_LOCAL_MACHINE\SOFTWARE\7-Zip
  < RegDelete Key=” HKEY_LOCAL_MACHINE\SOFTWARE\7-Zip”/>

2：刪除 HKEY_LOCAL_MACHINE\SOFTWARE\7-Zip 中的 Path 值
  <RegDelete Key=”HKEY_LOCAL_MACHINE\SOFTWARE\7-Zip” Value=”Path”/>
-->
```
### RegMove
移動登錄檔信息。
```xml
<element name="RegMove">
  <complexType>
    <!--來源 Key，信息將從這裡讀取，比如 "HKEY_LOCAL_MACHINE\SOFTWARE\Test"-->
    <attribute name="Key" type="字串" use="必選"/>
    <!--目的 Key，信息將被移動到這裡，比如 "HKEY_LOCAL_MACHINE\SOFTWARE\NewTest"-->
    <attribute name="NewKey" type="字串" use="必選"/>
    <!--如果存在此值，那麼僅從來源移動值信息-->
    <attribute name="Value" type="字串" use="選用"/>
    <!--新的值名稱。如果不存在此屬性那麼名稱保持不變。-->
    <attribute name="NewValue" type="字串" use="選用"/>
    <!--True 或者 False，指示是否開啟 Wow64 重定向，32 位元系統自動忽略-->
    <attribute name="Wow64" type="字串" use="選用" default="False"/>
  </complexType>
</element>
<!--
1：移動 7-zip 登錄檔
<RegMove Key=” HKEY_LOCAL_MACHINE\SOFTWARE\7-Zip”  NewKey=” HKEY_LOCAL_MACHINE\SOFTWARE\7-Zip2”/>
2：移動 7-zip 登錄檔的 Path 值為 OldPath
<RegMove Key=” HKEY_LOCAL_MACHINE\SOFTWARE\7-Zip”  NewKey=” HKEY_LOCAL_MACHINE\SOFTWARE\7-Zip2” Value=”Path” NewValue=”OldPath”/>
-->
```
### FileMove
用於移動檔案路徑。其路徑支援環境變數。
```xml
<element name="FileMove">
  <complexType>
    <!--來源檔案目錄-->
    <attribute name="Src" type="字串" use="必選"/>
    <!--目標檔案目錄-->
    <attribute name="Dst" type="字串" use="必選"/>
  </complexType>
</element>
<!--
1：將系統槽的 test.xml 移動到 test.bak
<FileMove Src="%SystemDrive%\test.xml" Dst="%SystemDrive%\test.bak"/>
-->
```

### FileCreateByZIP
從現有的 zip 檔案中解壓縮一個檔案釋放到指定目錄。
```xml
<element name="FileCreateByZIP">
  <complexType>
    <!--需要釋放的目錄-->
    <attribute name="Path" type="字串" use="必選"/>
    <!--zip 檔案目錄以及子目錄-->
    <attribute name="ZIPFile" type="字串" use="必選"/>
  </complexType>
</element>
<!--
釋放 Config\default.ui.zip 中的 Data\Blank.ico 檔案到 %systemroot%\Blank.ico
<FileCreateByZIP Path="%systemroot%\Blank.ico" ZIPFile="Config\default.ui.zip/Data/Blank.ico"/>
-->
```

### SetServiceStart
調整服務狀態
```xml
<element name="SetServiceStart">
  <complexType>
    <!--服務的名稱-->
    <attribute name="Name" type="字串" use="必選"/>
    <!--服務啟動類型，必須是以下值：
    0：驅動啟動項目
    1：系統啟動
    2：自動
    3：手動
    4：已停用
    5：自動 (延遲啟動)
  -->
    <attribute name="Type" type="整數" use="必選"/>
  </complexType>
</element>
<!--
將 aaa 服務調整為已停用
<SetServiceStart  Name=”aaa”  Type="4"/>
-->
```

### ExplorerNotify
通知系統設定更改。
```xml
<element name="ExplorerNotify">
  <complexType>
    <!--通知類型。只可能是 AssocChanged（更新檔案關聯）、Cmd（執行 Cmd 命令）、Custom（自訂）-->
    <attribute name="Type" type="字串" use="必選"/>
    <!--表示動作是否非同步進行，True 或者 False-->
    <attribute name="Async" type="邏輯值" use="選用" default="False"/>

    <!--需要執行的 cmd 命令，如果 Type 為 Cmd，那麼還允許以下屬性。-->
    <!--需要執行的 Cmd 命令-->
    <attribute name="Cmd" type="字串" use="必選"/>

    <!--如果 Type 是 Custom，那麼還允許以下屬性-->
    <!--消息類型-->
    <attribute name="msg" type="整數（16 進制）" use="必選"/>
    <!--WPARAM 參數，#開頭表示 16 進制
    #FF 一個 16 進制數
    TraySettings 字串，TraySettings
  -->
    <attribute name="wParam" type="字串/整數（16 進制）" use="選用" default="NULL"/>
    <!--LPARAM 參數，#開頭表示 16 進制
    #FF 一個 16 進制數
    TraySettings 字串，TraySettings
  -->
    <attribute name="lParam" type="字串/整數（16 進制）" use="選用" default="NULL"/>
    <!--是否僅通知 Explorer-->
    <attribute name="Explorer" type="邏輯值" use="選用" default="False"/>
  </complexType>
</element>
<!--
1：隱藏 Win10 6 個資料夾後 我們需要重新整理 Explorer 配置，可以這樣
<ExplorerNotify Type="Custom" msg="111" wParam="#A220" Explorer="true"/>

2：隱藏小盾牌後，重新整理檔案關聯以顯示新的圖示。
<ExplorerNotify Type="AssocChanged"/>

3：關閉 netprofm 服務
<ExplorerNotify Type="Cmd" Cmd=”net stop netprofm”/>

4：重新整理 Explorer 設定
<ExplorerNotify Type="Custom" msg="1A" lParam="TraySettings"/>
-->
```

### General
用於刪除指定特徵的檔案，在垃圾清理常用。

```xml
<element name="General">
  <complexType>
    <!--路徑名稱，支援增強函數，以及環境變數-->
    <attribute name="RootPath" type="字串" use="必選"/>
    <!--匹配類型 Directory 或者 File。-->
    <attribute name="Flags" type="字串" use="選用" default="Directory|File"/>

    <sequence>
      <!--要匹配的子路徑，如果沒有此節點，那麼表示匹配所有檔案-->
      <element name="Query" use="選用" default="*"/>
    </sequence>
    <sequence>
      <!--需要排除的字路徑。如果沒有此節點，那麼表示沒有排除列表。-->
      <element name="Excluded" use="選用"/>
    </sequence>
  </complexType>
</element>
<!--
將 C:\123 目錄清空（保留 123 這個資料夾）
<General RootPath= C:\123"/>

將 C:\123 目錄刪除（不保留 123 這個資料夾）
<General RootPath= C:\">
   < Query>123</Query >
</General>

將 C:\Temp 裡面的所有 *.txt 檔案刪除，但是保留裡面的 123.txt 以及 456.txt
<General RootPath= C:\ Temp" Flags=”File”>
    < Query >*.txt</ Query >
    < Excluded>123.txt</Excluded>
    < Excluded>456.txt</Excluded>
</General>
-->
```
### CScript
使用 C 腳本進行垃圾清理，具體語法規則，請參考 C 程式語言 C99 規範。（目前僅 Beta 版可用）
```xml
<element name="CScript">
  <complexType>
    <!--執行腳本函數名稱，其原型必須是
HRESULT WINAPI MyCleanup(DismSession Session, DWORD Flags, UINT64 *CleanUpSpace, DismCallBack CallBack, LPVOID UserData);
-->
    <attribute name="ProcName" type="字串" use="必選"/>
  </complexType>
</element>
<!--
< CScript ProcName =” MyCleanup”/>

然後在 XML 的 Data/ CScript 中添加函數代碼
<Data>
  <CScript>
    <![CDATA[
    #include <Dism++.h>
    HRESULT WINAPI MyCleanup(DismSession Session, DWORD Flags, UINT64 *CleanUpSpace, DismCallBack CallBack, LPVOID UserData)
    {
    If(CleanUpSpace)
    {
        //僅檢查可用清理項目
        //…
        //檢查完成後，返回可以清理的條目，單位 Byte
        * CleanUpSpace=1024;
    }
    else
    {
        //進行垃圾清理 開始清理垃圾
    }
    //返回 0 表示操作成功，其他任意值均表示操作失敗
        return 0;
    }
    ]]>
  </CScript>
<Data>
-->
```
### Custom
自訂清理動作，General 無法滿足條件時可以使用。需要自己編寫 dll。
```xml
<element name="Custom">
  <complexType>
    <!--
    需要呼叫函數的名稱
    其原型必須是
    HRESULT (WINAPI*)(DismSession Session, DWORD Flags, UINT64 *CleanUpSpace, DismCallBack CallBack, LPVOID UserData);
  -->
    <attribute name="ProcName" type="字串" use="必選"/>

	<!--10.1.2000.12以上版本可用-->
	<attribute name="Flags" type="十六進位整數" use="選用" default="#0"/>

  </complexType>
</element>

回調函數參數說明：
DismSession Session：Image 對話，可以使用此對話取得 Image 各種信息。
DWORD Flags：保留，Dism++ 現在不使用此參數，請忽略。
UINT64 *CleanUpSpace：如果 CleanUpSpace 為空，那麼函數這執行清理動作。如果不為空，說明僅預估可清理的空間。最後將預估大小用此變數返回。
DismCallBack CallBack：Dism++ 清理回掉函數，用於展示進度，檔案路徑等信息。如果此參數為 NULL，則表示沒有回調。
回調函數支援以下消息：
DISM_MSG_PROGRESS – 用於反饋處理進度，進度信息 wParam 為目前完成百分比，lParam 暫不使用，設定為 0 即可。
DISM_MSG_PROCESS – 用於在狀態欄中展示正在處理的檔案路徑，wParam = (PWSTR) pszFullPath，lParam 設定 0 即可。
DISM_MGS_RemoveInfo – 報告 UI 需要刪除的檔案，此消息僅掃描時可用，清理時將無視此消息。wParam 設定為 0，lParam=(LPCWSTR)FilePath 表示需要刪除的路徑。Dism++ 收到此消息後，會將檔案路徑展示在詳細信息中。

LPVOID UserData：回調函數的 UserData 部分，請務必傳入 CallBack 中。

回調函數返回值：
如果函數執行成功，請返回 S_OK，其他任何值都表示錯誤。

```
